@@ -16,6 +16,8 @@
 #define AR(str) rb_str_concat(buf, (str))
 
 #define A_INDENT add_indent(buf, indent)
+#define D_INDENT rb_str_cat2(indent, next_indent)
+#define D_DEDENT rb_str_resize(indent, RSTRING_LEN(indent) - 4)
 #define A_ID(id) add_id(buf, (id))
 #define A_INT(val) rb_str_catf(buf, "%d", (val))
 #define A_LONG(val) rb_str_catf(buf, "%ld", (val))
@@ -32,11 +34,16 @@
 #define COMPOUND_FIELD(len, name, block) \
     do { \
 	D_FIELD_HEADER((len), (name), "\n");	\
-	rb_str_cat2(indent, next_indent); \
+	D_INDENT; \
 	block; \
-	rb_str_resize(indent, RSTRING_LEN(indent) - 4); \
+	D_DEDENT; \
     } while (0)
 
+#define COMPOUND_FIELD1(name, ann, block) \
+    COMPOUND_FIELD(FIELD_NAME_LEN(name, ann), \
+		   FIELD_NAME_DESC(name, ann), \
+		   block)
+
 #define FIELD_NAME_DESC(name, ann) name " (" ann ")"
 #define FIELD_NAME_LEN(name, ann) (int)( \
 	comment ? \
@@ -57,9 +64,9 @@
 #define F_MSG(name, ann, desc)	    SIMPLE_FIELD1(#name, ann) A(desc)
 
 #define F_NODE(name, ann) \
-    COMPOUND_FIELD(FIELD_NAME_LEN(#name, ann), \
-		   FIELD_NAME_DESC(#name, ann), \
-		   dump_node(buf, indent, comment, node->name))
+    COMPOUND_FIELD1(#name, ann, dump_node(buf, indent, comment, node->name))
+#define F_OPTION(name, ann) \
+    COMPOUND_FIELD1(#name, ann, dump_option(buf, indent, node->name))
 
 #define ANN(ann) \
     if (comment) { \
@@ -91,11 +98,66 @@ add_id(VALUE buf, ID id)
     }
 }
 
+struct add_option_arg {
+    VALUE buf, indent;
+    st_index_t count;
+};
+
+static int
+add_option_i(VALUE key, VALUE val, VALUE args)
+{
+    struct add_option_arg *argp = (void *)args;
+    VALUE buf = argp->buf;
+    VALUE indent = argp->indent;
+
+    A_INDENT;
+    A("+- ");
+    AR(rb_sym2str(key));
+    A(": ");
+    A_LIT(val);
+    A("\n");
+    return ST_CONTINUE;
+}
+
+static void
+dump_option(VALUE buf, VALUE indent, VALUE opt)
+{
+    struct add_option_arg arg;
+
+    if (!RB_TYPE_P(opt, T_HASH)) {
+	A_LIT(opt);
+	return;
+    }
+    arg.buf = buf;
+    arg.indent = indent;
+    arg.count = 0;
+    rb_hash_foreach(opt, add_option_i, (VALUE)&arg);
+}
+
+static void dump_node(VALUE, VALUE, int, NODE *);
+static const char default_indent[] = "|   ";
+
+static void
+dump_array(VALUE buf, VALUE indent, int comment, NODE *node)
+{
+    int field_flag;
+    const char *next_indent = default_indent;
+    F_LONG(nd_alen, "length");
+    F_NODE(nd_head, "element");
+    while (node->nd_next && nd_type(node->nd_next) == NODE_ARRAY) {
+	node = node->nd_next;
+	F_NODE(nd_head, "element");
+    }
+    LAST_NODE;
+    F_NODE(nd_next, "next element");
+}
+
 static void
 dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 {
     int field_flag;
-    const char *next_indent = "|   ";
+    int i;
+    const char *next_indent = default_indent;
 
     if (!node) {
 	D_NULL_NODE;
@@ -107,9 +169,21 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
     switch (nd_type(node)) {
       case NODE_BLOCK:
 	ANN("statement sequence");
-	ANN("format: [nd_head]; [nd_next]");
+	ANN("format: [nd_head]; ...; [nd_next]");
 	ANN("example: foo; bar");
-	F_NODE(nd_head, "current statement");
+	i = 0;
+	do {
+	    A_INDENT;
+	    rb_str_catf(buf, "+- nd_head (%s%d):\n",
+			comment ? "statement #" : "", ++i);
+	    if (!node->nd_next) LAST_NODE;
+	    D_INDENT;
+	    dump_node(buf, indent, comment, node->nd_head);
+	    D_DEDENT;
+	} while (node->nd_next &&
+		 nd_type(node->nd_next) == NODE_BLOCK &&
+		 (node = node->nd_next, 1));
+	if (!node->nd_next) break;
 	LAST_NODE;
 	F_NODE(nd_next, "next block");
 	break;
@@ -447,10 +521,7 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 	ANN("format: [ [nd_head], [nd_next].. ] (length: [nd_alen])");
 	ANN("example: return 1, 2, 3");
       ary:
-	F_LONG(nd_alen, "length");
-	F_NODE(nd_head, "element");
-	LAST_NODE;
-	F_NODE(nd_next, "next element");
+	dump_array(buf, indent, comment, node);
 	break;
 
       case NODE_ZARRAY:
@@ -542,8 +613,12 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
         ANN("format: [nd_recv] =~ [nd_value]");
 	ANN("example: /foo/ =~ 'foo'");
 	F_NODE(nd_recv, "regexp (receiver)");
-	LAST_NODE;
+	if (!node->nd_args) LAST_NODE;
 	F_NODE(nd_value, "string (argument)");
+	if (node->nd_args) {
+	    LAST_NODE;
+	    F_NODE(nd_args, "named captures");
+	}
 	break;
 
       case NODE_MATCH3:
@@ -824,11 +899,14 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 	ANN("pre-execution");
 	ANN("format: BEGIN { [nd_head] }; [nd_body]");
 	ANN("example: bar; BEGIN { foo }");
-	F_NODE(nd_head, "prelude");
-	F_NODE(nd_body, "body");
-	LAST_NODE;
 #define nd_compile_option u3.value
-	F_LIT(nd_compile_option, "compile_option");
+	F_NODE(nd_head, "prelude");
+	if (!node->nd_compile_option) LAST_NODE;
+	F_NODE(nd_body, "body");
+	if (node->nd_compile_option) {
+	    LAST_NODE;
+	    F_OPTION(nd_compile_option, "compile_option");
+	}
 	break;
 
       case NODE_LAMBDA:
@@ -883,8 +961,8 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 	F_ID(nd_ainfo->rest_arg, "rest argument");
 	F_ID(nd_ainfo->block_arg, "block argument");
 	F_NODE(nd_ainfo->opt_args, "optional arguments");
-	LAST_NODE;
 	F_NODE(nd_ainfo->kw_args, "keyword arguments");
+	LAST_NODE;
 	F_NODE(nd_ainfo->kw_rest_arg, "keyword rest argument");
 	break;
 
@@ -978,6 +1056,7 @@ rb_gc_mark_node(NODE *obj)
       case NODE_RESBODY:
       case NODE_CLASS:
       case NODE_BLOCK_PASS:
+      case NODE_MATCH2:
 	rb_gc_mark(RNODE(obj)->u2.value);
 	/* fall through */
       case NODE_BLOCK:	/* 1,3 */
@@ -1008,7 +1087,6 @@ rb_gc_mark_node(NODE *obj)
       case NODE_DOT3:
       case NODE_FLIP2:
       case NODE_FLIP3:
-      case NODE_MATCH2:
       case NODE_MATCH3:
       case NODE_OP_ASGN_OR:
       case NODE_OP_ASGN_AND:
